@@ -55,9 +55,10 @@
 #define MOD_GM_COMMON_H
 
 /* constants */
-#define GM_VERSION                 "5.1.5.20241105"
+#define GM_VERSION                 "5.1.8"
 #define GM_ENABLED                      1
 #define GM_DISABLED                     0
+#define GM_AUTO                         2
 #define GM_BUFFERSIZE               65536
 #define GM_SMALLBUFSIZE              1024
 #define GM_MAX_OUTPUT            10485760   /* limit plugin output size to 10mb */
