@@ -53,8 +53,8 @@ static pthread_mutex_t mod_gm_result_list_mutex = PTHREAD_MUTEX_INITIALIZER;
 static pthread_mutex_t mod_gm_log_lock = PTHREAD_MUTEX_INITIALIZER;
 void *gearman_module_handle=NULL;
 
-int result_threads_running;
-pthread_t result_thr[GM_LISTSIZE];
+int gm_should_terminate = FALSE;
+pthread_t * result_thr;
 char target_queue[GM_SMALLBUFSIZE];
 char temp_buffer[GM_MAX_OUTPUT];
 char uniq[GM_SMALLBUFSIZE];
@@ -92,10 +92,13 @@ static void  move_results_to_core(struct nm_event_execution_properties *evprop);
 static int   handle_hst_check_result(int event_type, void *data);
 static int   handle_svc_check_result(int event_type, void *data);
 static int   try_check_dummy(const char *, host *, service * );
+void shutdown_threads(void);
+void process_check_result_list(void);
+static int start_threads(void);
 
 int nebmodule_init( int flags, char *args, nebmodule *handle ) {
     int broker_option_errors = 0;
-    result_threads_running   = 0;
+    gm_should_terminate = FALSE;
 
     /* save our handle */
     gearman_module_handle=handle;
@@ -179,21 +182,21 @@ int nebmodule_init( int flags, char *args, nebmodule *handle ) {
     }
     current_client = client;
 
+    if(start_threads() != GM_OK) {
+        gm_log( GM_LOG_ERROR, "failed to start result threads\n" );
+        return NEB_ERROR;
+    }
+
     /* register callback for process event where everything else starts */
     neb_register_callback(NEBCALLBACK_PROCESS_DATA,        gearman_module_handle, 0, handle_process_events );
     neb_register_callback(NEBCALLBACK_PROGRAM_STATUS_DATA, gearman_module_handle, 0, handle_progam_status_data_events);
     schedule_event(1, move_results_to_core, NULL);
 
     /* log at least one line into the core logfile */
-    if ( mod_gm_opt->logmode != GM_LOG_MODE_CORE ) {
-        int logmode_saved = mod_gm_opt->logmode;
-        mod_gm_opt->logmode = GM_LOG_MODE_CORE;
-        if(strlen(GIT_HASH) > 0)
-            gm_log( GM_LOG_INFO,  "initialized version %s (build: %s) (libgearman %s)\n", GM_VERSION, GIT_HASH, gearman_version() );
-        else
-            gm_log( GM_LOG_INFO,  "initialized version %s (libgearman %s)\n", GM_VERSION, gearman_version() );
-        mod_gm_opt->logmode = logmode_saved;
-    }
+    if(strlen(GIT_HASH) > 0)
+        nm_log( NSLOG_INFO_MESSAGE, "mod_gearman: initialized version %s (build: %s) (libgearman %s)\n", GM_VERSION, GIT_HASH, gearman_version() );
+    else
+        nm_log( NSLOG_INFO_MESSAGE, "mod_gearman: initialized version %s (libgearman %s)\n", GM_VERSION, gearman_version() );
 
     gm_log( GM_LOG_DEBUG, "finished initializing\n" );
 
@@ -236,8 +239,9 @@ static void register_neb_callbacks(void) {
 
 /* deregister all events */
 int nebmodule_deinit( int flags, int reason ) {
-    int x;
+    int x = 0;
 
+    nm_log( NSLOG_INFO_MESSAGE, "mod_gearman: deinitializing\n" );
     gm_log( GM_LOG_TRACE, "nebmodule_deinit(%i, %i)\n", flags, reason );
 
     /* should be removed already, but just for the case it wasn't */
@@ -265,18 +269,17 @@ int nebmodule_deinit( int flags, int reason ) {
     }
 
     /* register export callbacks */
-    for(x=0;x<GM_NEBTYPESSIZE;x++) {
+    for(x = 0; x < GM_NEBTYPESSIZE; x++) {
         if(mod_gm_opt->exports[x]->elem_number > 0)
             neb_deregister_callback( x, gearman_module_handle );
     }
 
     gm_log( GM_LOG_DEBUG, "deregistered callbacks\n" );
 
-    /* stop result threads */
-    for(x = 0; x < result_threads_running; x++) {
-        pthread_cancel(result_thr[x]);
-        pthread_join(result_thr[x], NULL);
-    }
+    shutdown_threads();
+
+    // clean check result list
+    process_check_result_list();
 
     /* cleanup */
     gm_free_client(&client);
@@ -292,19 +295,45 @@ int nebmodule_deinit( int flags, int reason ) {
     return NEB_OK;
 }
 
+void shutdown_threads() {
+    int x = 0;
+
+    gm_should_terminate = TRUE;
+    if(result_thr == NULL) {
+        return;
+    }
+
+    /* stop result threads */
+    for(x = 0; x < mod_gm_opt->result_workers; x++) {
+        if(pthread_cancel(result_thr[x]) != OK) {
+            gm_log( GM_LOG_ERROR, "failed to join cancel thread: %s\n", strerror(errno) );
+        }
+        if(pthread_join(result_thr[x], NULL) != OK) {
+            gm_log( GM_LOG_ERROR, "failed to join result thread: %s\n", strerror(errno) );
+        }
+    }
+    gm_free(result_thr);
+    result_thr = NULL;
+}
+
 /* insert results list into naemon core */
 static void move_results_to_core(struct nm_event_execution_properties *evprop) {
-    struct timeval tval_before, tval_after, tval_result;
-    objectlist *tmp_list = NULL;
-    objectlist *cur = NULL;
-    int count = 0;
     if(evprop->execution_type != EVENT_EXEC_NORMAL) {
         return;
     }
 
+    process_check_result_list();
+    schedule_event(1, move_results_to_core, NULL);
+}
+
+void process_check_result_list() {
+    objectlist *tmp_list = NULL;
+    objectlist *cur = NULL;
+    struct timeval tval_before, tval_after, tval_result;
+    int count = 0;
+
     gettimeofday(&tval_before, NULL);
     gm_log( GM_LOG_TRACE3, "move_results_to_core()\n" );
-    schedule_event(1, move_results_to_core, NULL);
 
     /* safely move result list aside */
     pthread_mutex_lock(&mod_gm_result_list_mutex);
@@ -320,7 +349,10 @@ static void move_results_to_core(struct nm_event_execution_properties *evprop) {
         cur = tmp_list;
         tmp_list = tmp_list->next;
 
-        process_check_result(cur->object_ptr);
+        // simply clean the results, we cannot put them back to core anymore
+        if(gm_should_terminate == FALSE)
+            process_check_result(cur->object_ptr);
+
         free_check_result(cur->object_ptr);
         free(cur->object_ptr);
         free(cur);
@@ -342,15 +374,22 @@ void mod_gm_add_result_to_list(check_result * newcheckresult) {
 }
 
 /* start our threads */
-static void start_threads(void) {
-    if ( result_threads_running < mod_gm_opt->result_workers ) {
-        /* create result worker */
-        int x;
-        for(x = 0; x < mod_gm_opt->result_workers; x++) {
-            result_threads_running++;
-            pthread_create ( &result_thr[x], NULL, result_worker, (void *)&result_threads_running);
+static int start_threads(void) {
+    int x;
+    int ret = 0;
+    if( mod_gm_opt->result_workers <= 0 ) {
+        return(GM_OK);
+    }
+    result_thr = gm_malloc(mod_gm_opt->result_workers * sizeof *result_thr);
+
+    /* create result worker */
+    for(x = 0; x < mod_gm_opt->result_workers; x++) {
+        if((ret = pthread_create(&result_thr[x], NULL, &result_worker, NULL)) != OK) {
+            gm_log( GM_LOG_ERROR, "failed to create result thread: %s\n", strerror(ret));
+            return(GM_ERROR);
         }
     }
+    return(GM_OK);
 }
 
 /* handle process events */
@@ -376,12 +415,15 @@ static int handle_process_events( int event_type, void *data ) {
     gm_log( GM_LOG_TRACE, "handle_process_events(%i, data)\n", event_type );
 
     ps = ( struct nebstruct_process_struct * )data;
+    if(ps->type == NEBTYPE_PROCESS_EVENTLOOPEND ) {
+        shutdown_threads();
+        return NEB_OK;
+    }
     if(ps->type != NEBTYPE_PROCESS_EVENTLOOPSTART ) {
         return NEB_OK;
     }
 
     register_neb_callbacks();
-    start_threads();
 
     /* verify names of supplied groups
         * this cannot be done befor naemon has finished reading his config
@@ -680,9 +722,9 @@ static int handle_notifications( int event_type, void *data ) {
 
     /* set the notification id macro */
     if(svc != NULL) {
-        gm_asprintf(&mac.x[MACRO_SERVICENOTIFICATIONID], "%lu", svc->current_notification_id);
+        gm_asprintf(&mac.x[MACRO_SERVICENOTIFICATIONID], "%s", svc->current_notification_id);
     } else {
-        gm_asprintf(&mac.x[MACRO_HOSTNOTIFICATIONID], "%lu", hst->current_notification_id);
+        gm_asprintf(&mac.x[MACRO_HOSTNOTIFICATIONID], "%s", hst->current_notification_id);
     }
 
     /* process any macros contained in the argument */
